#!/usr/bin/python
from chi2_regular_binning import *
from chi2_adaptive_binning import *
from chi2_plots import *
import os

def example_call():
	orig_name="gaussian_same_projection_redefined__0_1__0_75"
	title="Gauss 0.1 0.75"
	dim_list = [1,2,3]
	systematics_fraction = 0.01

	comp_file_list_list = []
	adaptive_binning=False
	for dim_data in dim_list:
		comp_file_list=[]
		for i in range(1):
			comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_10000_0.0_1.0_1.0_{0}.txt".format(i,dim_data),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_10000_0.0_1.0_0.75_{0}.txt".format(i,dim_data)))
		comp_file_list_list.append(comp_file_list)
	if adaptive_binning==True:
		number_of_splits_list = [1,2]
		#chi2_adaptive_binning.chi2_adaptive_binning(orig_name, dim_list, comp_file_list_list,number_of_splits_list)
		chi2_adaptive_binning_wrapper(title,orig_name, dim_list, comp_file_list_list,number_of_splits_list,systematics_fraction)
	else:
		single_no_bins_list=[2,3,5]
		chi2_regular_binning_wrapper(title,orig_name, dim_list, comp_file_list_list,single_no_bins_list,systematics_fraction)	

if __name__ == "__main__":
	example_call()



import numpy as np
#import matplotlib
#from matplotlib.pylab import *
import matplotlib.pyplot as plt
import matplotlib.colors
from matplotlib.patches import Rectangle
import matplotlib.cm as cm
import time


def histo_plot_pvalue(U_0,abins,axlabel,aylabel,atitle,aname):
        bins_probability=np.histogram(U_0,bins=abins)[1]

        #Finding the p values corresponding to 1,2 and 3 sigma significance.
        no_one_std_dev=sum(i < (1-0.6827) for i in U_0)
        no_two_std_dev=sum(i < (1-0.9545) for i in U_0)
        no_three_std_dev=sum(i < (1-0.9973) for i in U_0)

        print(no_one_std_dev,no_two_std_dev,no_three_std_dev)

        with open(aname+"_p_values_1_2_3_std_dev.txt",'w') as p_value_1_2_3_std_dev_file:
                p_value_1_2_3_std_dev_file.write(str(no_one_std_dev)+'\t'+str(no_two_std_dev)+'\t'+str(no_three_std_dev)+'\n')

        #plt.rc('text', usetex=True)
        textstr = '$1\sigma=%i$\n$2\sigma=%i$\n$3\sigma=%i$'%(no_one_std_dev, no_two_std_dev, no_three_std_dev)


        # Making a histogram of the probability predictions of the algorithm. 
        fig_pred_0= plt.figure()
        ax1_pred_0= fig_pred_0.add_subplot(1, 1, 1)
        n0, bins0, patches0 = ax1_pred_0.hist(U_0, bins=bins_probability, facecolor='red', alpha=0.5)
        ax1_pred_0.set_xlabel(axlabel)
        ax1_pred_0.set_ylabel(aylabel)
        ax1_pred_0.set_title(atitle+ " p values")
        plt.xlim([0,1])

        # these are matplotlib.patch.Patch properties
        props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)

        # place a text box in upper left in axes coords
        ax1_pred_0.text(0.85, 0.95, textstr, transform=ax1_pred_0.transAxes, fontsize=14,
        verticalalignment='top', bbox=props)

        fig_pred_0.savefig(aname+"_p_values_plot.png")
        #plt.close(fig_pred_0)

def adaptive_binning_2Dplot(bin_boundaries_dict,signed_Scp2_dict,number_of_splits,X_values,title,name):
	if X_values.shape[1]==2:
		fig= plt.figure()
		ax1= fig.add_subplot(1, 1, 1)	
		signed_Scp2_values = []
		for bin_key, bin_Scp2 in signed_Scp2_dict.items(): 
			if str(number_of_splits) in bin_key: 
				signed_Scp2_values.append(bin_Scp2)	
		norm = matplotlib.colors.Normalize(min(signed_Scp2_values),max(signed_Scp2_values))
		my_cmap = cm.get_cmap('Greys')
		
		ax1.scatter(X_values[:,0], X_values[:,1], c='black',s=5, lw = 0)

		for bin_key, bin_boundary in bin_boundaries_dict.items():
			if str(number_of_splits) in bin_key:
				#print("Boundaries : ",bin_boundary[0,0], bin_boundary[1,0], bin_boundary[0,1],bin_boundary[1,1])
				cd=ax1.add_patch(Rectangle((bin_boundary[0,0], bin_boundary[1,0]),bin_boundary[0,1]-bin_boundary[0,0] , bin_boundary[1,1]-bin_boundary[1,0],alpha=0.5, edgecolor="black",facecolor = my_cmap(norm(signed_Scp2_dict[bin_key])),linewidth=2))
	
		plt.xlim(0.0,1.0)
		plt.ylim(0.0,1.0)	
                plt.xlabel("normalised x feature")
                plt.ylabel("normalised y feature")
                plt.title(name + " bin definitions" )
		fig.savefig(name+"_bin_definitions_2D.png")
		print("The plot {}_bin_definitions_2D.png has been made".format(name))
	else:
		print("You can only make this plot in 2D")


def adaptive_binning_1Dplot(bin_boundaries_dict,data,number_of_splits,title,name):
	if data.shape[1]-1==1:
		data0    = data[data[:,-1]==0]
                data1    = data[data[:,-1]==1]
		#print("data0 : ",data0)
		features_0 = data0[:,:-1]
		features_1 = data1[:,:-1]
		#print("features_0 : ",features_0)

		fig= plt.figure()
		ax1= fig.add_subplot(1, 1, 1)
		#bin_sample01_list = []
		bin_boundaries = []
		bin_boundaries_lower = []
		for bin_key, bin_boundaries_in_dict in bin_boundaries_dict.items():
			if str(number_of_splits) in bin_key:
				#bin_sample01_list.append(bins_sample01)
				bin_boundaries.append(bin_boundaries_in_dict[0][1])
				bin_boundaries_lower.append(bin_boundaries_in_dict[0][0])

		bin_boundaries = sorted(bin_boundaries)
                #bin_boundaries, features = zip(*sorted(zip(bin_boundaries,bin_sample01_list)))
		#print("bin_boundaries : ",bin_boundaries)
		bin_boundaries = [min(bin_boundaries_lower)] + list(bin_boundaries)
		#print("bin_boundaries : ",bin_boundaries)

		#print("bin_sample01_list : ",np.array(bin_sample01_list))

		#bin_sample0 = np.array(bin_sample01_list)[:,0]
                #bin_sample1 = np.array(bin_sample01_list)[:,1]
		#print("features_0.shape : ",features_0.shape,"\nfeatures_1.shape : ",features_1.shape)
		plt.hist(features_0,bins=bin_boundaries,color='red',label='original',alpha=0.4,normed=True)
		plt.hist(features_1,bins=bin_boundaries,color='blue',label= 'modified',alpha=0.4,normed=True)
                
		plt.xlabel("normalised feature")
                plt.title(title + " bin definitions" )
		fig.savefig(name+"_bin_definitions_1D.png")
                print("The plot {}_bin_definitions_1D.png has been made".format(name))
        else:
                print("You can only make this plot in 1D")
		

def regular_binning_2Dplot(keys, signed_Scp2,single_no_bins,X_values,title,name):
        no_dim = X_values.shape[1]
	if no_dim==2:
		no_bins = [single_no_bins]*no_dim

                fig= plt.figure()
                ax1= fig.add_subplot(1, 1, 1)
                norm = matplotlib.colors.Normalize(np.min(signed_Scp2),np.max(signed_Scp2))
                my_cmap = cm.get_cmap('Greys')

                ax1.scatter(X_values[:,0], X_values[:,1], c='black',s=5, lw = 0)

		for key_index, key in enumerate(keys):
			key_list = np.array(key.split(',')).astype(int).tolist()
 

			bin_boundary = np.array([[float(key_list[0])/float(single_no_bins)  ,float(key_list[0]+1)/float(single_no_bins) ],[float(key_list[1])/float(single_no_bins)  ,float(key_list[1]+1)/float(single_no_bins)  ]])
			print("bin_boundary : ",bin_boundary)
			print("bin_boundary[0,0] : ",bin_boundary[0,0])
			print("signed_Scp2 : ", signed_Scp2)


			cd=ax1.add_patch(Rectangle((bin_boundary[0,0], bin_boundary[1,0]),bin_boundary[0,1]-bin_boundary[0,0] , bin_boundary[1,1]-bin_boundary[1,0],alpha=0.5, edgecolor="black",facecolor = my_cmap(norm(signed_Scp2[key_index])),linewidth=2))

                plt.xlim(0,1)
                plt.ylim(0,1)
                plt.xlabel("normalised x feature")
		plt.ylabel("normalised y feature")
                plt.title(title + " bin definitions" )

		fig.savefig(name+"_bin_definitions_2D.png")
                print("The plot {}_bin_definitions_2D.png has been made".format(name))
        else:
                print("You can only make this plot in 2D")

def regular_binning_1Dplot(data,single_no_bins,title,name):
        if data.shape[1]-1==1:
                data0    = data[data[:,-1]==0]
                data1    = data[data[:,-1]==1]
                #print("data0 : ",data0)
                features_0 = data0[:,:-1]
                features_1 = data1[:,:-1]

                fig= plt.figure()
                ax1= fig.add_subplot(1, 1, 1)

                plt.hist(features_0,bins=np.linspace(0,1,single_no_bins+1),color='red',label='original',alpha=0.4,normed=True)
                plt.hist(features_1,bins=np.linspace(0,1,single_no_bins+1),color='blue',label= 'modified',alpha=0.4,normed=True)
		plt.xlabel("normalised feature")
		plt.title(title + " bin definitions" )

                fig.savefig(name+"_bin_definitions_1D.png")
                print("The plot {}_bin_definitions_1D.png has been made".format(name))
        else:
                print("You can only make this plot in 1D")





























import sys 
import chi2_plots
import itertools

"""
This script can be used to get the p value for the Miranda method (=chi squared). It takes input files with column vectors corresponding to 
features and lables. 
"""

print(__doc__)
import sys
#sys.path.insert(0,'../..')
import os
from scipy import stats
import numpy as np
#import matplotlib.pyplot as plt 
#import numpy.matlib
#from matplotlib.colors import Normalize

#from sklearn.preprocessing import StandardScaler

##############################################################################
# Setting parameters
#
#orig_name= sys.argv[1]
#single_no_bins_list_list= ast.literal_eval(sys.argv[2])
#dim_list = ast.literal_eval(sys.argv[3])
#comp_file_list_list = ast.literal_eval(sys.argv[4])

def chi2_regular_binning_wrapper(orig_title, orig_name, dim_list, comp_file_list_list,single_no_bins_list,systematics_fraction):

	sample1_name="original"
	sample2_name="modified"

	DEBUG = True


	##############################################################################
	for dim_index, dim_data in enumerate(dim_list):
		print("We are now in "+str(dim_data) + " Dimensions")
		#comp_file_list=[]
		comp_file_list = comp_file_list_list[dim_index]
		#for i in range(100):

			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/higher_dimensional_gauss/gauss_data/data_high" +str(dim_data)+"Dgauss_10000_0.5_0.1_0.0_{0}.txt".format(i),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/higher_dimensional_gauss/gauss_data/data_high"+str(dim_data)+"Dgauss_10000_0.5_0.1_0.01_{0}.txt".format(i))) 
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/legendre/legendre_data/data_legendre_contrib0__1__10__sample_{0}.txt".format(i),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/legendre/legendre_data/data_legendre_contrib0__1__9__sample_{0}.txt".format(i)))
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.1_{0}.txt".format(i,dim_data),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.075_{0}.txt".format(i,dim_data)))
		#print(comp_file_list)

		for single_no_bins in single_no_bins_list:
			score_list=[]
			counter = 0
			for comp_file_0,comp_file_1 in comp_file_list:
				name = orig_name + "_" +str(dim_data) + "D_chi2_" + str(single_no_bins)+ "_bins"
				title= orig_title +" " +str(dim_data) + "D "      + str(single_no_bins)+ " bins"
				print("single_no_bins : ",single_no_bins)
				print("Operating of files :"+comp_file_0+"   "+comp_file_1)

				#extracts data from the files
				features_0=np.loadtxt(comp_file_0,dtype='d')
				features_1=np.loadtxt(comp_file_1,dtype='d')

				result = chi2_regular_binning(features_0,features_1,single_no_bins,systematics_fraction,title,name,not counter,DEBUG)	
				score_list.append(result)
				counter+=1


                        with open(name+"_p_values", "wb") as test_statistics_file:
                                for score in score_list:
                                        test_statistics_file.write(str(score)+"\n")


			#if dim_data==2: os.rename(str(dim_data) + "D_" + str(single_no_bins)+"_bins" +"_bin_definitions_2D.png",name+"_bin_definitions_2D.png")
                        #if dim_data==1: os.rename(str(dim_data) + "D_" + str(single_no_bins)+"_bins" +"_bin_definitions_1D.png",name+"_bin_definitions_1D.png")
                        chi2_plots.histo_plot_pvalue(score_list,50,"p value","Frequency",title,name)


def chi2_regular_binning(features_0,features_1,single_no_bins,systematics_fraction=0.0,title= "title", name="name", PLOT = True, DEBUG = False):
		#determine how many data points are in each sample
		no_0=features_0.shape[0]
		no_1=features_1.shape[0]

		#Give all samples in file 0 the label 0 and in file 1 the feature 1
		label_0=np.zeros((no_0,1))
		label_1=np.ones((no_1,1))

		#Create an array containing samples and features.
		data_0=np.c_[features_0,label_0]
		data_1=np.c_[features_1,label_1]

		data=np.r_[data_0,data_1]

		no_dim = data.shape[1]-1
		no_bins = [single_no_bins]*no_dim       

		np.random.shuffle(data)

		labels=data[:,-1]

		X_values= data[:,:-1]
		X_max   = np.amax(data,axis=0)[:-1] 
		X_min   = np.amin(data,axis=0)[:-1]
		X_total_width = (np.subtract(X_max,X_min))
		X_width = (np.divide(np.subtract(X_max,X_min),no_bins))
		del data

		#Scaling        
		X_values = X_values - X_min[None,:]
		X_values = X_values / X_total_width[None,:]

		#b = X_values[:,0]
		#print("b[b[:]>2].shape[0] : \n", b[b[:]>2].shape[0] )  
		data = np.concatenate((X_values, labels[:,None]), axis=1)

		bin_points_dict = {}

		key_list = [list(p) for p in itertools.product(range(single_no_bins), repeat=no_dim)]
		#for c in combinations_with_replacement(range(single_no_bins), no_dim):
		for c in key_list:
			bin_points_dict[','.join(map(str,c))]= [0,0]

		print("bin_points_dict : \n",bin_points_dict)

		for i in range(no_0+no_1):
			#bin position
			#x_bin=int(np.floor((Xx_values[i]-Xx_min)/Xx_width))
			#y_bin=int(np.floor((Xy_values[i]-Xy_min)/Xy_width))

			pos_bins=np.floor(X_values[i,:]*no_bins).astype(int)
			pos_bins[pos_bins[:]==single_no_bins]=single_no_bins-1
			#print(pos_bins)

			if(labels[i]==0): 
				bin_points_dict[','.join(map(str,pos_bins))][0]+=1
			if(labels[i]==1):
				bin_points_dict[','.join(map(str,pos_bins))][1]+=1

		print("bin_points_dict : \n",bin_points_dict)
		

		Scp2 =  []
		signed_Scp2 = []
		keys = []

		for bin_key, bin_entry in bin_points_dict.items():
			bin_sample0 = bin_entry[0]
			bin_sample1 = bin_entry[1]
			if(systematics_fraction*float(bin_sample0)!=0.): bin_sample0 += int(round(np.random.normal(0.,systematics_fraction*float(bin_sample0))))
			if(systematics_fraction*float(bin_sample1)!=0.): bin_sample1 += int(round(np.random.normal(0.,systematics_fraction*float(bin_sample1))))
			bin_points_dict[bin_key] = [bin_sample0, bin_sample1]				

			keys.append(bin_key)
			Scp2.append(np.square(float(bin_sample1-bin_sample0))/(float(bin_sample1)+float(bin_sample0)+np.square(float(bin_sample1)*systematics_fraction)+np.square(float(bin_sample1)*systematics_fraction)))
			signed_Scp2.append(np.square(float(bin_sample1-bin_sample0))/(float(bin_sample1)+float(bin_sample0)+np.square(float(bin_sample1)*systematics_fraction)+np.square(float(bin_sample1)*systematics_fraction))*np.sign(bin_sample1-bin_sample0))

		keys,Scp2,signed_Scp2 = zip(*sorted(zip(keys,Scp2,signed_Scp2)))



		#nansum ignores all the contributions that are Not A Number (NAN)
		Chi2 = np.nansum(Scp2)
		if DEBUG:
			print("Chi2")
			print(Chi2)
		dof=no_bins[0]
		for dim in range(1,no_dim):
			dof *= no_bins[1]
		dof-=1
	
		pvalue= 1 - stats.chi2.cdf(Chi2,dof)

		if DEBUG:
			print("Chi2/dof : {0}".format(str(Chi2/dof)))

			print("pvalue : {0}".format(str(pvalue)))

		if PLOT:
				if no_dim==1: chi2_plots.regular_binning_1Dplot(data,single_no_bins,title,name)
				if no_dim==2: chi2_plots.regular_binning_2Dplot(keys, signed_Scp2,single_no_bins,X_values,title,name)

		return pvalue










from __future__ import print_function
import sys 
import chi2_plots 
import random
import ast
"""
This script can be used to get the p value for the Miranda method (=chi squared). It takes input files with column vectors corresponding to 
features and lables. 
"""

print(__doc__)
import sys
#sys.path.insert(0,'../..')
import os
from scipy import stats
import numpy as np
#import matplotlib.pyplot as plt 
#import numpy.matlib
#from matplotlib.colors import Normalize

#from sklearn.preprocessing import StandardScaler

##############################################################################
# Setting parameters
#
#orig_name= sys.argv[1]
#number_of_splits_list= ast.literal_eval(sys.argv[2])
#print("number_of_splits_list : ", number_of_splits_list)
#dim_list = ast.literal_eval(sys.argv[3])
#comp_file_list_list = ast.literal_eval(sys.argv[4])


def chi2_adaptive_binning_wrapper(orig_title, orig_name, dim_list, comp_file_list_list,number_of_splits_list,systematics_fraction):

	sample1_name="original"
	sample2_name="modified"

	DEBUG = False


	##############################################################################
	for dim_index, dim_data in enumerate(dim_list):
		print("We are now in "+str(dim_data) + " Dimensions")
		#comp_file_list=[]
		comp_file_list = comp_file_list_list[dim_index]
		#for i in range(2):

			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/higher_dimensional_gauss/gauss_data/data_high" +str(dim_data)+"Dgauss_10000_0.5_0.1_0.0_{0}.txt".format(i),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/higher_dimensional_gauss/gauss_data/data_high"+str(dim_data)+"Dgauss_10000_0.5_0.1_0.01_{0}.txt".format(i))) 
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/legendre/legendre_data/data_legendre_contrib0__1__10__sample_{0}.txt".format(i),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/legendre/legendre_data/data_legendre_contrib0__1__9__sample_{0}.txt".format(i)))
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.1_{0}.txt".format(i,dim_data),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.075_{0}.txt".format(i,dim_data)))
		#print(comp_file_list)

		score_dict = {}
		for number_of_splits in number_of_splits_list:
			score_dict[str(number_of_splits)]=[]
		
		counter = 0
		for comp_file_0,comp_file_1 in comp_file_list:
			print("Operating of files :"+comp_file_0+"   "+comp_file_1)

			#extracts data from the files
			features_0=np.loadtxt(comp_file_0,dtype='d')
			features_1=np.loadtxt(comp_file_1,dtype='d')

			#only make a plot for the first data set
			results_list=chi2_adaptive_binning(features_0,features_1,number_of_splits_list,systematics_fraction,orig_title,orig_name, not counter,DEBUG)
			for number_of_splits_index, number_of_splits in enumerate(number_of_splits_list):
				score_dict[str(number_of_splits)].append(results_list[number_of_splits_index])	

			counter+=1

                for number_of_splits in number_of_splits_list:
                        name = orig_name + "_" +str(dim_data) + "D_chi2_" + str(number_of_splits) + "_splits"
                        title= orig_title+ " " +str(dim_data) + "D "      + str(number_of_splits) + " splits" 
			print("score_dict[{}] : ".format(number_of_splits), score_dict[str(number_of_splits)])
                        with open(name+"_adaptive_binning_p_values", "wb") as test_statistics_file:
                                for score in score_dict[str(number_of_splits)]:
                                        test_statistics_file.write(str(score)+"\n")
			#if dim_data==2: os.rename("name_"+str(dim_data) + "D_" + str(number_of_splits) + "_splits"+"_bin_definitions_2D.png",name+"_bin_definitions_2D.png")
                        #if dim_data==1: os.rename("name_"+str(dim_data) + "D_" + str(number_of_splits) + "_splits"+"_bin_definitions_1D.png",name+"_binning_bin_definitions_1D.png")
                        chi2_plots.histo_plot_pvalue(score_dict[str(number_of_splits)],50,"p value","Frequency",title,name)





def chi2_adaptive_binning(features_0,features_1,number_of_splits_list,systematics_fraction=0.0,title = "title", name="name", PLOT = True, DEBUG = False):
	"""This function takes in two 2D arrays with all features being columns"""

	max_number_of_splits = np.max(number_of_splits_list)
	#determine how many data points are in each sample
	no_0=features_0.shape[0]
	no_1=features_1.shape[0]

	#Give all samples in file 0 the label 0 and in file 1 the feature 1
	label_0=np.zeros((no_0,1))
	label_1=np.ones((no_1,1))

	#Create an array containing samples and features.
	data_0=np.c_[features_0,label_0]
	data_1=np.c_[features_1,label_1]

	data=np.r_[data_0,data_1]

	no_dim = data.shape[1]-1

	np.random.shuffle(data)

	#print("data.shape : ", data.shape)
	labels=data[:,-1]

	X_values= data[:,:-1]
	X_max   = np.amax(data,axis=0)[:-1] 
	X_min   = np.amin(data,axis=0)[:-1]
	X_total_width = (np.subtract(X_max,X_min))
	del data

	#Scaling	
	X_values = X_values - X_min[None,:]
	X_values = X_values / X_total_width[None,:]

	#b = X_values[:,0]
	#print("b[b[:]>2].shape[0] : \n", b[b[:]>2].shape[0] )	
	data = np.concatenate((X_values, labels[:,None]), axis=1)

	#print("X_values.shape : ",X_values.shape)
	starting_boundary = []
	for i in range(no_dim):
		starting_boundary.append([0.0,1.0])
	#Each key has the following stricture: # of splits and for each split if it was closer (a) or further away from (b) the origin. The original bin is "0"
	#For example "2ab" means it is the bin that was closer to the origin for the first split and further away for the second one.
	bin_boundaries_dict = {'0' : np.array(starting_boundary)}
	bin_points_dict = {'0' : data}

	for split_number in range(1,1+max_number_of_splits):
		for bin_key, bin_boundary in bin_boundaries_dict.items():
			if str(split_number-1) in bin_key:
				variances= np.var(bin_points_dict[bin_key][:,:-1], axis=0)
				#print("\nvariances : ", variances)
				dim_to_be_sliced = np.argmax(variances)
				#print("dim_to_be_sliced : ",dim_to_be_sliced)
				#print("bin_points_dict[bin_key] : ",bin_points_dict[bin_key])
				#print("bin_points_dict[bin_key][:,dim_to_be_sliced] : ",bin_points_dict[bin_key][:,dim_to_be_sliced])
				median = np.median(bin_points_dict[bin_key][:,dim_to_be_sliced])
				#print("median : ",median)

				a_bin_boundary, b_bin_boundary = bin_boundary.copy(), bin_boundary.copy()
				#print("a_bin_boundary : ",a_bin_boundary)
				a_bin_boundary[dim_to_be_sliced,1] = median
				b_bin_boundary[dim_to_be_sliced,0] = median
				bin_boundaries_dict[str(split_number)+bin_key[1:]+'a'] = a_bin_boundary
				bin_boundaries_dict[str(split_number)+bin_key[1:]+'b'] = b_bin_boundary
				
				a_points, b_points = [],[] 
				for event_number in range(bin_points_dict[bin_key].shape[0]):
					if bin_points_dict[bin_key][event_number,dim_to_be_sliced] < median: a_points.append(bin_points_dict[bin_key][event_number,:].tolist())
					else: b_points.append(bin_points_dict[bin_key][event_number,:].tolist())
				
				bin_points_dict[str(split_number)+bin_key[1:]+'a'] = np.array(a_points)
				bin_points_dict[str(split_number)+bin_key[1:]+'b'] = np.array(b_points)
				#If a bin contains no particles it should be deleted
				if len(a_points)==0: 
					del bin_points_dict[str(split_number)+bin_key[1:]+'a']
					del bin_boundaries_dict[str(split_number)+bin_key[1:]+'a']
				if len(b_points)==0: 
					del bin_points_dict[str(split_number)+bin_key[1:]+'b']
					del bin_boundaries_dict[str(split_number)+bin_key[1:]+'b']

		
	bins_sample01_dict= {}
	signed_Scp2_dict= {}
	results_list = []

	for number_of_splits in number_of_splits_list:
		print("\nnumber_of_splits : ",number_of_splits,"\nsystematics_fraction : ",systematics_fraction)
		bins_sample0, bins_sample1 = [] , [] 
		for bin_key, bin_points in bin_points_dict.items():
			if str(number_of_splits) in bin_key:
				labels_in_bin = bin_points[:,-1]
				#print("labels_in_bin : ",labels_in_bin)
				bin_sample0 = np.count_nonzero( labels_in_bin == 0)
				bin_sample1 = np.count_nonzero( labels_in_bin == 1)
				#print("bin_sample0 : ",bin_sample0) 
				#print("bin_sample1 : ",bin_sample1) 
				#simulate uncertainties 
				if(systematics_fraction*float(bin_sample0)!=0.): bin_sample0 += int(round(np.random.normal(0.,systematics_fraction*float(bin_sample0))))
				if(systematics_fraction*float(bin_sample1)!=0.): bin_sample1 += int(round(np.random.normal(0.,systematics_fraction*float(bin_sample1))))
				bins_sample01_dict[bin_key]=[bin_sample0,bin_sample1]
				signed_Scp2_dict[bin_key] = np.square(float(bin_sample1-bin_sample0))/(float(bin_sample1)+float(bin_sample0)+np.square(float(bin_sample1)*systematics_fraction)+np.square(float(bin_sample1)*systematics_fraction))*np.sign(bin_sample1-bin_sample0)
				#print("\n\nbin_sample0 : ",bin_sample0, "\n bins_sample0 : ", bins_sample0 )
				#print("type(bin_sample0) : ",type(bin_sample0), "  type(bins_sample0) : ",type(bins_sample0))
				bins_sample0.append(bin_sample0)
				#print("  bins_sample0 : ", bins_sample0, "\n\n" )
				bins_sample1.append(bin_sample1)
		bins_sample0, bins_sample1 = np.array(bins_sample0,dtype=float), np.array(bins_sample1, dtype=float)
		print("bins_sample0 : ",bins_sample0,"\n bins_sample1 : ",bins_sample1)
		#element wise subtraction and division
		Scp2 = ((bins_sample1-bins_sample0)**2)/ (bins_sample1+bins_sample0+(systematics_fraction*bins_sample1)**2+(systematics_fraction*bins_sample0)**2 )
		#Scp2 =  np.divide(np.square(np.subtract(bins_sample1,bins_sample0)),np.add(bins_sample1,bins_sample0))
		if DEBUG:
			print(Scp2)

		#nansum ignores all the contributions that are Not A Number (NAN)
		Chi2 = np.nansum(Scp2)
		if DEBUG:
			print("Chi2")
			print(Chi2)
		dof=bins_sample0.shape[0]-1
	
		pvalue= 1 - stats.chi2.cdf(Chi2,dof)
		
		print("\nThe p value for Scp2 = ",Scp2," and Chi2 = ", Chi2, " is ",pvalue,"\n\n")
		if DEBUG:
			print(bins_sample0)
			print(bins_sample1)
			print("Chi2/dof : {0}".format(str(Chi2/dof)))

			print("pvalue : {0}".format(str(pvalue)))

		results_list.append(pvalue)

		if PLOT: 
			if no_dim==1: chi2_plots.adaptive_binning_1Dplot(bin_boundaries_dict,data,number_of_splits,title+" "+str(no_dim) + "D "+str(number_of_splits)+ " splits ",name+"_"+str(no_dim) + "D_chi2_"+str(number_of_splits)+"_splits")
			if no_dim==2: chi2_plots.adaptive_binning_2Dplot(bin_boundaries_dict,signed_Scp2_dict,number_of_splits,X_values,title+" "+str(no_dim) + "D"+str(number_of_splits)+ " splits ",name+"_"+str(no_dim) + "D_chi2_"+str(number_of_splits)+"_splits")

	return results_list 









import sys
import os
os.environ["adaptive_binning_chisquared_2sam"]=os.getcwd()
#sys.path.insert(0, './GoF')

from chi2_regular_binning import *
from chi2_adaptive_binning import *
from chi2_plots import *
from calling_chi2_example import *

